
from Interface import Base

class IHostOverride( Base ):
    """An optional part of a virtual host that is given the
    opportunity to override host actions"""

    def testActivate(host, stack, stackString=""):
        """Tests whether this override should be called.  host is full
        host name (e.g. 'test.at.myhost.com'), stack is actual stack
        from request (or, for testing, in same format), and
        stackString is stack reversed and with slashes to make it look
        like the original path from the URI."""

    def activate(testResponse, stack, VHFid, request, path, addressList, matchedStack=[]):
        """Acts on the request (usually the stack).  testResponse is
        whatever testActivate returned (to allow for not doing the
        same work twice, if applicable).  Stack is the actual request
        stack (or, for testing, in same format) possibly modified by
        parent host overrides to place matched elements in the
        matchedStack (see below), VHFid is the id of the host folder
        (so it can be inserted in the stack as necessary, request is
        the request object (or reasonable testing facsimile), path is
        the virtual host root path as determined by the parent virtual
        host, addressList is the current order of hosts and host
        overrides that have been called so far, and matchedStack is
        a list of elements of the stack that have been removed by parent
        hostOverrides so far.  Return nothing or an updated stackString
        (that is, if you return a string it will replace the current
        stackString) to continue processing host overrides; return -1 to
        stop checking the overrides and have the host put itself on the
        stack; or any other value to stop all further processing by the
        host (so the override is responsible for setting the stack and
        setting the VIRTUAL_HOST_IDS)."""


class ISimpleAliasHostOverride( IHostOverride ):
    """Represents one alias, an optional part of a virtual host.  Holds alias
    information, and can act as a dynamically-inserted pseudo-folder in
    traversal."""

    def getVirtualEquivalentPath(host, pathFromRoot, URLFromRoot):
        """Given a string of the host and a path (list) and URL
        (string) from the Zope root, tries to return a path from the
        virtual root that will use this override or one of its
        children to reach the same location."""

    def getTraversalResponse():
        """A hook that the VHF uses to give the host override an
        opportunity to provide a traversal object or list of
        objects"""

class IAliasHostOverride( ISimpleAliasHostOverride ):
    """All the Alias getters and setters."""
    
    def setIncludeDestinationPath(setting, mfunc=None):
        """sets where the destination path to the alias should be
        included in the stack, if anywhere, as determined by bit-map
        settings held in the constants DESTINATION_SHOWN_LAST,
        DESTINATION_SHOWN_FIRST, DESTINATION_HIDDEN_LAST, and
        DESTINATION_HIDDEN_FIRST, or simply 0 (off).  If mfunc is
        included, it may be called with explanatory messages (usually
        mfunc is an array append)."""

    def getIncludeDestinationPath():
        """returns 0 (no destination path to be included) or one of
        the values from the four constants listed above."""

    def setForceAlias(setting, mfunc=None):
        """sets if the alias object should itself be inserted in the
        stack, to attempt to force automatically generated links to
        keep the alias in the path.  Settings are held in constants
        FORCE_ACTIVE_ALIAS and FORCE_PASSIVE_ALIAS, or simply 0 (off).
        If mfunc is included, it may be called with explanatory
        messages (usually mfunc is an array append)."""

    def getForceAlias():
        """gets one of the settings listed above"""

    def setIncludeParents(setting, mfunc=None):
        """Sets whether the original parents of the alias should be
        included in the stack (useful to try and retain path
        information in automatically generated links).  Any
        boolean-equivalent true/false setting will work.  If you are
        including parents, you must include host root as well.  If
        mfunc is included, it may be called with explanatory messages
        (usually mfunc is an array append)."""

    def getIncludeParents():
        """Gets the setting described above, returning either 0 (off)
        or a positive integer (on)."""

    def setIncludeHostRoot(setting, mfunc=None):
        """Sets whether the host root will be included in the stack; a
        necessity for 'setIncludeParents', and useful if you want the
        destination object to be able to acquire any information as if
        it were in the actual virtual host path.  Any
        boolean-equivalent true/false setting will work.  If mfunc is
        included, it may be called with explanatory messages (usually
        mfunc is an array append)."""

    def getIncludeHostRoot():
        """Gets the setting described above, returning either 0 (off)
        or a positive integer (on)."""

    def setRetainPathAfterAlias(setting, mfunc=None):
        """Sets whether the path after alias should be retained in the
        stack.  If it is not retained, the alias' index_html will
        always be returned for any call down this path.  Any
        boolean-equivalent true/false setting will work.  If mfunc is
        included, it may be called with explanatory messages (usually
        mfunc is an array append)."""

    def getRetainPathAfterAlias():
        """Gets the setting described above, returning either 0 (off)
        or a positive integer (on)."""

    def setParentsAndAliasName(parent, aliasName, mfunc=None):
        """Sets the parent of the alias (that is, the path from the
        virtual host root in which the alias should appear to reside),
        and the aliasName that, during traversal, will appear to be
        the name of a folder in the parent.  If mfunc is included, it
        may be called with explanatory messages (usually mfunc is an
        array append)."""

    def getAliasName():
        """returns the aliasName, as described above."""

    def getParent():
        """returns the parent, as described above."""

    def setMapping(mapping, mfunc=None):
        """Sets the destination folder to which the alias should map,
        as found from the Zope root. If mfunc is included, it may be
        called with explanatory messages (usually mfunc is an array
        append)."""

    def getMapping():
        """returns the mapping, as described above."""

    def getFullURL():
        """A calculated convenience--a string combining the parent and
        the alias name to look like a full path."""

    def getFullPath():
        """A calculated convenience--a list of strings, indicating
        each step along the fullURL"""

    def getStatus(mfunc=None):
        """Returns 0 (alias is inactive, for whatever reason) or a
        positive integer (alias is on) that is a bitmap indicating the
        current alias options.  If mfunc is included, it will be
        called with a message that attempts to interpret for what
        purpose the current alias settings might be intended."""

class IAliasHostOverrideContainer( Base ):
    """containers of an AliasHostOverride must implement this
    behavior"""

    def checkParentsAndAliasName(parent, aliasName, caller, mfunc=None):
        """called by child aliases when the alias parent or aliasName
        is modified.  caller is the child caller, mfunc is the
        standard optional message-passing function.  returns 1 if
        aliasName and path are acceptable as far as the container is
        concerned, 0 if not."""

class ISimpleRedirectHostOverride( IHostOverride ):
    """Represents one redirect, an optional part of a virtual host.  Holds
    redirect information, and calls a redirect within BeforeTraverse if there is a match."""

    def activate(testResponse, stack=None, VHFid="", request={}, path="", addressList=[], matchedStack=[]):
        """overrides the IHostOverride interface signature to make
        everything optional except the testResponse."""

class IRedirectHostOverride( ISimpleRedirectHostOverride ):
    def setRegexTemplate(self, tmpl, mfunc=None):
        """sets the template; supports the optional
        messaging system."""

    def getRegexTemplate():
        """returns the unprocessed template"""

    def setRegex(regex, ignorecase=1, mfunc=None):
        """sets the domain regular expression.  Pass in anything that evaluates
        to Boolean true in ignorecase to set it to true.  No other flags are
        available (although they could be added, I didn't feel they would be
        useful).  The messages array is an optional human-readable
        messaging system."""

    def getRegex():
        """returns the current regular expression *object*"""

    def getRegexComponents():
        """returns the current regular expression and the ignorecase
        flag setting.  If no current regular expression is set, returns an empty string and the default 'on' ignorecase setting."""

class ISimpleStackRewriter( IHostOverride ):
    """A simple way to rewrite the path stack.  No additional methods needed."""

class IStackRewriter( ISimpleStackRewriter ):
    """A simple way to rewrite the path stack.  Needs to define getters and setters."""